import pytest

from aiovcf.utils import fbool, fint, to_typed_value


def test_to_typed_value():
    assert to_typed_value("5", int) == 5
    assert to_typed_value("5.0", fint) == pytest.approx(5.0)
    assert to_typed_value("xxy", int) is None
    assert to_typed_value("xxy", fint) is None


def test_fint():
    assert fint(None) is None
    assert fint("5x") is None
    assert fint("x5") is None
    assert fint("3.14159") == pytest.approx(3.14159)
    assert fint("3") == 3


def test_fbool():
    assert fbool(False) is False
    assert fbool(None) is False
    assert fbool(0) is False
    assert fbool(0.0) is False
    assert fbool("0") is False
    assert fbool("0.0") is False
    assert fbool("") is False

    assert fbool(True) is True
    assert fbool(1) is True
    assert fbool(-1) is True
    assert fbool(1.0) is True
    assert fbool(-1.0) is True
    assert fbool("1.0") is True
    assert fbool("abc") is True

    # odd balls (not doing literal eval...)
    assert fbool("None") is True
    assert fbool("False") is True
    assert fbool("True") is True



import related

from aiovcf.models import (
    Field,
    FieldIndex,
    Genotype,
    Meta,
    Record,
    SampleRecord,
    SamplesIndex,
    Value,
)
from aiovcf.parser import Parser


def test_format_field():
    assert Parser.parse(
        '##FORMAT=<ID=GQ,Number=1,Type=Integer,Description="Genotype Quality">'
    ) == Field(
        section="FORMAT",
        id="GQ",
        number="1",
        type="Integer",
        description="Genotype Quality",
    )


def test_info_field():
    assert Parser.parse(
        '##INFO=<ID=AF,Number=A,Type=Float,Description="Allele Frequency">'
    ) == Field(
        section="INFO",
        id="AF",
        number="A",
        type="Float",
        description="Allele Frequency",
    )


def test_contig_ignored():
    actual = Parser.parse(
        "##contig=<ID=20,length=62435964,assembly=B36,"
        'md5=f126cdf8a6e0c7f379d618ff66beb2da,species="Homo sapiens",'
        "taxonomy=x> "
    )

    expected = Field(
        section="contig",
        id="20",
        number=None,
        type=None,
        description=None,
        other=[
            Value(name="length", value=62435964),
            Value(name="assembly", value="B36"),
            Value(name="md5", value="f126cdf8a6e0c7f379d618ff66beb2da"),
            Value(name="species", value="Homo sapiens"),
            Value(name="taxonomy", value="x"),
        ],
    )

    assert related.to_json(expected) == related.to_json(actual)


def test_meta():
    assert Parser.parse("##fileformat=VCFv4.1") == Meta(
        name="fileformat", value="VCFv4.1"
    )


def test_samples():
    assert Parser.parse(
        "#CHROM	POS	ID	REF	ALT	QUAL	FILTER	INFO	FORMAT	"
        "NA00001	NA00002	NA00003"
    ) == SamplesIndex(samples=["NA00001", "NA00002", "NA00003"])


def test_record():
    field_index = FieldIndex()
    field_index.sample.add(
        Parser.parse(
            '##FORMAT=<ID=AO,Number=A,Type=Integer,Description="Alternate '
            'allele observation count"> '
        )
    )

    samples_index = Parser.parse(
        "#CHROM	POS	ID	REF	ALT	QUAL	FILTER	INFO	FORMAT	NA00001"
        "	NA00002	NA00003"
    )

    actual = Parser.parse(
        "21	4242421	.	T	A	30	.	.	GT:AO	0|0:0.1	0|1:0.2	0/0:0.3",
        field_index=field_index,
        samples_index=samples_index,
    )

    # . means not None, not just []
    assert actual.filters is None

    expected = Record(
        chromosome="21",
        position=4242421,
        id=None,
        ref="T",
        alts=["A"],
        quality=30,
        filters=None,
        info={},
        format=["GT", "AO"],
        samples=[
            SampleRecord(
                sample_id="NA00001",
                genotype=Genotype(value="0|0", alleles={0}, is_phased=True),
                info={"AO": Value(name="AO", value=0.1)},
            ),
            SampleRecord(
                sample_id="NA00002",
                genotype=Genotype(value="0|1", alleles={0, 1}, is_phased=True),
                info={"AO": Value(name="AO", value=0.2)},
            ),
            SampleRecord(
                sample_id="NA00003",
                genotype=Genotype(value="0/0", alleles={0}, is_phased=False),
                info={"AO": Value(name="AO", value=0.3)},
            ),
        ],
    )

    # related default is overriding None
    expected.filters = None

    expect = related.to_json(expected)
    actual = related.to_json(actual)

    assert expect == actual, f"Mismatch:\n\n{expect}\n\n{actual}"



# from aiovcf import reader
import os

import pytest
import related

from aiovcf import iter_file, iter_calls

INPUT_VCF = os.path.join(os.path.dirname(__file__), "input_vcf")
OUTPUT_RECS = os.path.join(os.path.dirname(__file__), "output_recs")
OUTPUT_CALLS = os.path.join(os.path.dirname(__file__), "output_calls")


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "input_vcf",
    [
        os.path.join(INPUT_VCF, fname)
        for fname in sorted(os.listdir(INPUT_VCF))
    ],
)
async def test_iter_file(input_vcf):
    assert os.path.isfile(input_vcf)
    filename = os.path.basename(input_vcf)[:-4]

    # check records

    output_recs = os.path.join(OUTPUT_RECS, f"{filename}.json")
    assert os.path.isfile(output_recs), f"Output recs is missing: {filename}"

    all_records = []
    async for records in iter_file(input_vcf):
        all_records.append(records)

    actual = related.to_json(all_records, suppress_empty_values=True)
    expect = open(output_recs).read().strip()
    assert expect == actual, f"{filename} Mismatch:\n{actual}\n"


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "input_vcf",
    [
        os.path.join(INPUT_VCF, fname)
        for fname in sorted(os.listdir(INPUT_VCF))
    ],
)
async def test_iter_calls(input_vcf):
    assert os.path.isfile(input_vcf)
    filename = os.path.basename(input_vcf)[:-4]

    # check calls

    output_calls = os.path.join(OUTPUT_CALLS, f"{filename}.json")
    assert os.path.isfile(output_calls), f"Output calls is missing: {filename}"

    all_calls = []
    async for call in iter_calls(input_vcf):
        all_calls.append(call)

    actual = related.to_json(all_calls, suppress_empty_values=True)
    expect = open(output_calls).read().strip()
    assert expect == actual, f"{filename} Mismatch:\n\n{actual}"



from aiovcf.models import Record


def test_add_filter():
    record = Record()
    assert record.filters == []

    record.add_filter(None)
    assert record.filters is None

    record.add_filter("PASS")
    assert record.filters == []

    record.add_filter("j123")
    assert record.filters == ["j123"]

    record.add_filter("k456")
    assert record.filters == ["j123", "k456"]

    record.add_filter("PASS")
    assert record.filters == ["j123", "k456"]

    record.add_filter(None)
    assert record.filters == ["j123", "k456"]
